/* DM_Comm_Initialize is the C file that contains all of the initialization
functions needed to run the DM_Comm.c file.
Written by Barrett Taylor on July 3, 2013.
*/

#include "DM_Headers.h"

uint8_t g_uDMAControlTable[1024] __attribute__ ((aligned(1024)));
uint16_t g_ROERxBufA[8];
uint16_t g_ROERxBufB[8];
/**********************************************************************************************
void InitializeClock(): This function is used to set the system clock of the board to the
desired values.
**********************************************************************************************/
void InitializeClock(void)
{
	//Set the system clock to run from the PLL and set to run at 66.6MHz. Set the main oscillator to 16MHz.
	SysCtlClockSet(SYSCTL_SYSDIV_3 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN | SYSCTL_XTAL_16MHZ);
}

/**********************************************************************************************
void Init_uDMA_Controller(void (* pfnHandler )(void));: This function is used to configure
the uDMA controller.
**********************************************************************************************/
void Init_uDMA_Controller()
{
	SysCtlPeripheralEnable(SYSCTL_PERIPH_UDMA);																	//Enable the uDMA peripheral.
	IntEnable(INT_UDMAERR);																											//Enable the uDMA error interrupt.
	uDMAEnable();																																//Enable the uDMA to allow configuration.
	uDMAControlBaseSet(&g_uDMAControlTable[0]);																	//Give the address of the control table parameters.
}

/**********************************************************************************************
void InitializeSSI(): This function is used to initialize SSI0 over port A pins. This sets up
the Master SSI to send data to the OSIRIS.
**********************************************************************************************/
void InitializeSSI(void)
{
	SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);				//Enable GPIO port A pins which are used for SSI0.
	SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOD);				//Enable GPIO port D pins which are used for SSI3.
	SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOF);				//Enable GPIO port F pins which are used for SSI1.
	
	SysCtlPeripheralEnable(SYSCTL_PERIPH_SSI0);					//Enable SSI0 so the clock can be configured.
	SysCtlPeripheralEnable(SYSCTL_PERIPH_SSI1);					//Enable SSI1 so the clock can be configured.
	SysCtlPeripheralEnable(SYSCTL_PERIPH_SSI3);					//Enable SSI3 so the clock can be configured.
	
	SSIClockSourceSet(SSI0_BASE, SSI_CLOCK_SYSTEM);			//Use the system clock as the clock for the SSI0.
	SSIClockSourceSet(SSI1_BASE, SSI_CLOCK_SYSTEM);			//Use the system clock as the clock for the SSI1.
	SSIClockSourceSet(SSI3_BASE, SSI_CLOCK_SYSTEM);			//Use the system clock as the clock for the SSI3.
	
	GPIOPinConfigure(GPIO_PA2_SSI0CLK);									//Configure the four GPIO pins to be used for the SSI0.
	GPIOPinConfigure(GPIO_PA3_SSI0FSS);
	GPIOPinConfigure(GPIO_PA4_SSI0RX);
	GPIOPinConfigure(GPIO_PA5_SSI0TX);
	
	GPIOPinConfigure(GPIO_PF2_SSI1CLK);									//Configure the four GPIO pins to be used for the SSI1.
	GPIOPinConfigure(GPIO_PF3_SSI1FSS);
	GPIOPinConfigure(GPIO_PF0_SSI1RX);
	GPIOPinConfigure(GPIO_PF1_SSI1TX);
	
	GPIOPinConfigure(GPIO_PD0_SSI3CLK);									//Configure the four GPIO pins to be used for the SSI3.
	GPIOPinConfigure(GPIO_PD1_SSI3FSS);
	GPIOPinConfigure(GPIO_PD2_SSI3RX);
	GPIOPinConfigure(GPIO_PD3_SSI3TX);
	
	//Configure pins for use by the SSI0.
	GPIOPinTypeSSI(GPIO_PORTA_BASE, GPIO_PIN_2 | GPIO_PIN_3 | GPIO_PIN_4 | GPIO_PIN_5);
	
	//Configure pins for use by the SSI1.
	GPIOPinTypeSSI(GPIO_PORTF_BASE, GPIO_PIN_2 | GPIO_PIN_3 | GPIO_PIN_0 | GPIO_PIN_1);
	
	//Configure pins for use by the SSI3.
	GPIOPinTypeSSI(GPIO_PORTD_BASE, GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_3);
	
	//Set the SSI0 (OCE Comm) as a slave and clock it from the system clock previously defined and run at 4MHz with 16 bit words.
	SSIConfigSetExpClk(SSI0_BASE, SysCtlClockGet(), SSI_FRF_MOTO_MODE_2, SSI_MODE_SLAVE, 4000000, 16);

	//Set the SSI1 (DM Tx) as a slave and clock it from the system clock previously defined and run at 2MHz with 16 bit words.
	SSIConfigSetExpClk(SSI1_BASE, SysCtlClockGet(), SSI_FRF_MOTO_MODE_2, SSI_MODE_SLAVE, 2000000, 16);
	
	//Set the SSI3 as a slave (DM Rx) and its clock to run from the system clock at 2MHz with 16 bit words.
	SSIConfigSetExpClk(SSI3_BASE, SysCtlClockGet(), SSI_FRF_MOTO_MODE_2, SSI_MODE_SLAVE, 2000000, 16);
	
	SSIEnable(SSI0_BASE);																//Enable the SSI0.
	SSIEnable(SSI1_BASE);																//Enable the SSI1.
	SSIEnable(SSI3_BASE);																//Enable the SSI3.
	
	SSIDMAEnable(SSI3_BASE, SSI_DMA_RX);

	IntEnable(INT_SSI3);
}

/**********************************************************************************************
void Configure_uDMA(): This functions configures the uDMA after it has been initialized and 
also after the SSI has been set up.
**********************************************************************************************/
void Configure_uDMA(void)
{
	uDMAChannelAssign(UDMA_CH14_SSI3RX);																					//Assign channel 14 to SSI0_Rx.
	uDMAChannelAttributeDisable(14, UDMA_ATTR_ALTSELECT | UDMA_ATTR_USEBURST
														| UDMA_ATTR_HIGH_PRIORITY | UDMA_ATTR_REQMASK);  		//Disable attributes on channel 14 to set to a known state.
	uDMAChannelAttributeEnable(14, UDMA_ATTR_USEBURST | UDMA_ATTR_HIGH_PRIORITY);	//Use burst mode (hard set at 4).
	uDMAChannelControlSet(14 | UDMA_PRI_SELECT, UDMA_SIZE_16 | 
													UDMA_SRC_INC_NONE | UDMA_DST_INC_16 | UDMA_ARB_4);		//Set the control words for channel 14 primary buffer.
	uDMAChannelControlSet(14 | UDMA_ALT_SELECT, UDMA_SIZE_16 | 
													UDMA_SRC_INC_NONE | UDMA_DST_INC_16 | UDMA_ARB_4);		//Set the control words for channel 14 alternate buffer.
	uDMAChannelTransferSet(14 | UDMA_PRI_SELECT, UDMA_MODE_PINGPONG,
													(void *) (SSI3_BASE + SSI_O_DR), g_ROERxBufA, 4);			//Set the transfer characteristics for channel 14 primary buffer.
	uDMAChannelTransferSet(14 | UDMA_ALT_SELECT, UDMA_MODE_PINGPONG,
													(void *) (SSI3_BASE + SSI_O_DR), g_ROERxBufB, 4);			//Set the transfer characteristics for channel 14 alternate buffer.
	uDMAChannelEnable(14);
}
/**********************************************************************************************
void Init_TIMER(): This function is used to configure the TIMER.
**********************************************************************************************/
void Init_TIMER (void) 
{
	SysCtlPeripheralEnable(SYSCTL_PERIPH_TIMER3);																//Enable TIMER3 peripheral function.
	SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);																//Enable port B peripheral function.
	GPIOPinConfigure(GPIO_PB2_T3CCP0);																					//Configure pin PB2 to be a timer.
	GPIOPinTypeTimer(GPIO_PORTB_BASE, GPIO_PIN_2);															//Specify the pin PB2 as a timer pin.
	TimerConfigure(TIMER3_BASE, TIMER_CFG_SPLIT_PAIR | TIMER_CFG_A_PWM);				//Configure timer 3 to use timer A on this port and output a pulse ot the pin.
	TimerLoadSet(TIMER3_BASE, TIMER_A, 32);																			//Start/reset the timer to 32 every timer (divides down to ~2MHz).
	TimerMatchSet(TIMER3_BASE, TIMER_A, 16);																		//Set the match case (toggle case) to half the set value for 50% duty cycle.
	TimerEnable(TIMER3_BASE, TIMER_A);																					//Enable the timer to start it running.
	
	SysCtlPeripheralEnable(SYSCTL_PERIPH_TIMER2);																//Enable the TIMER2 peripheral function.
	SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);																//Enable port B for peripheral function.
	GPIOPinConfigure(GPIO_PB0_T2CCP0);																					//Configure pin PB0 to be a timer.
	GPIOPinTypeTimer(GPIO_PORTB_BASE, GPIO_PIN_0);															//Specify pin PB0 as a timer pin.
	IntMasterEnable();																													//Enable the interrupts.
	TimerConfigure(TIMER2_BASE, TIMER_CFG_SPLIT_PAIR | TIMER_CFG_A_CAP_COUNT);	//Configure timer 2 to use timer A and be an input capture counter.
	TimerControlEvent(TIMER2_BASE, TIMER_A, TIMER_EVENT_POS_EDGE);							//Configure the capture event to be positive icoming edges.
	TimerLoadSet(TIMER2_BASE, TIMER_A, 14);																			//Set the timer to the desired start count.
	TimerMatchSet(TIMER2_BASE, TIMER_A, 0);																			//Set the match condition to the value where the counter ends and triggers the interrupt.
	IntEnable(INT_TIMER2A);																											//Enable the timer 2 interrupt for timer A.
	TimerIntEnable(TIMER2_BASE, TIMER_CAPA_MATCH);															//Enable and configure the timer interrupt to be in capture mode.

	SysCtlPeripheralEnable(SYSCTL_PERIPH_WTIMER0);															//Enable wide timer 0 (32 bit).
	SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOC);																//Enable port C peripheral function.
	GPIOPinConfigure(GPIO_PC5_WT0CCP1);																					//Configure pin PC5 to be a timer.
	GPIOPinTypeTimer(GPIO_PORTC_BASE, GPIO_PIN_5);															//Specify pin PC5 as a timer pin.
	TimerConfigure(WTIMER0_BASE, TIMER_CFG_SPLIT_PAIR | TIMER_CFG_B_CAP_COUNT);	//Configure the timer to count on timer B.
	TimerControlEvent(WTIMER0_BASE, TIMER_B, TIMER_EVENT_POS_EDGE);							//Configure the timer to count positive edges.
	TimerLoadSet(WTIMER0_BASE, TIMER_B, 0x01312D00);														//Set the timer to take about 10 seconds (running off of a 2MHz clock).
	TimerMatchSet(WTIMER0_BASE, TIMER_B, 0);																		//Set the match constant to 0 so that it counts the whole way down before interrupting.														
	IntEnable(INT_WTIMER0B);																										//Enable the interrupt for this timer.
	TimerIntEnable(WTIMER0_BASE, TIMER_CAPB_MATCH);															//Enable the interrupt to trigger on the match constant.

	SysCtlPeripheralEnable(SYSCTL_PERIPH_TIMER1);																//Enable the TIMER4 peripheral.
	SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);																//Enable port C peripheral function.
	GPIOPinConfigure(GPIO_PB5_T1CCP1);																					//Configure pin PC0 to be a timer.
	GPIOPinTypeTimer(GPIO_PORTB_BASE, GPIO_PIN_5);															//Specify pin PC0 as a timer pin.
	TimerConfigure(TIMER1_BASE, TIMER_CFG_SPLIT_PAIR | TIMER_CFG_B_CAP_COUNT);	//Configure the timer to count on timer A.
	TimerControlEvent(TIMER1_BASE, TIMER_B, TIMER_EVENT_POS_EDGE);							//Configure the timer to count positive edges.
	TimerLoadSet(TIMER1_BASE, TIMER_B, 4);																			//Count to FFFF (max count available).
	TimerMatchSet(TIMER1_BASE, TIMER_B, 0);																			//Set the match constant as 0 (when to fire interrupt).
	IntEnable(INT_TIMER1B);																											//Enable TIMER4 interrupt.
	TimerIntEnable(TIMER1_BASE, TIMER_CAPB_MATCH);															//Enable the interrupt to trigger on the match constant.
}

/**********************************************************************************************
void Init_GPIO(): This function is used to configure the GPIO ports to inputs/outputs, sets 
directions, internal resistors, etc.
**********************************************************************************************/
void Init_GPIO (void) 
{
	volatile unsigned long delay_clk; 															//Delay for clock, must have 3 sys clock delay.
	SYSCTL_RCGC2_R |= 0x00000002;																		//Enable the system clock to PORT B.
	delay_clk = SYSCTL_RCGC2;																				//Dummy read to ensure the clock has stabilized.
	GPIOPinTypeGPIOOutput(GPIO_PORTB_BASE, GPIO_PIN_3 | GPIO_PIN_4);//Make pin PB3 & PB4 be an output pin.
	GPIOPinWrite(GPIO_PORTB_BASE, GPIO_PIN_3, 0x08);								//Start pin 3 (DA line) high.
	GPIOPinWrite(GPIO_PORTB_BASE, GPIO_PIN_4, 0x00);								//Start pin 4 (OCE_DA line) low.
}
